@@ -17,13 +17,15 @@
 use ln::msgs;
 use ln::{PaymentPreimage, PaymentHash, PaymentSecret};
 use chain::keysinterface::SpendableOutputDescriptor;
-use util::ser::{Writeable, Writer, MaybeReadable, Readable};
+use util::ser::{Writeable, Writer, MaybeReadable, Readable, VecReadWrapper, VecWriteWrapper};
 
 use bitcoin::blockdata::script::Script;
 
 use bitcoin::secp256k1::key::PublicKey;
 
-use std::time::Duration;
+use prelude::*;
+use core::time::Duration;
+use core::ops::Deref;
 
 /// An Event which you should probably take some action in response to.
 ///
@@ -95,8 +97,6 @@ pub enum Event {
 	},
 	/// Indicates an outbound payment we made succeeded (ie it made it all the way to its target
 	/// and we got back the payment preimage for it).
-	/// Note that duplicative PaymentSent Events may be generated - it is your responsibility to
-	/// deduplicate them by payment_preimage (which MUST be unique)!
 	PaymentSent {
 		/// The preimage to the hash given to ChannelManager::send_payment.
 		/// Note that this serves as a payment receipt, if you wish to have such a thing, you must
@@ -105,8 +105,6 @@ pub enum Event {
 	},
 	/// Indicates an outbound payment we made failed. Probably some intermediary node dropped
 	/// something. You may wish to retry with a different route.
-	/// Note that duplicative PaymentFailed Events may be generated - it is your responsibility to
-	/// deduplicate them by payment_hash (which MUST be unique)!
 	PaymentFailed {
 		/// The hash which was given to ChannelManager::send_payment.
 		payment_hash: PaymentHash,
@@ -128,7 +126,8 @@ pub enum Event {
 		/// now + 5*time_forwardable).
 		time_forwardable: Duration,
 	},
-	/// Used to indicate that an output was generated on-chain which you should know how to spend.
+	/// Used to indicate that an output which you should know how to spend was confirmed on chain
+	/// and is now spendable.
 	/// Such an output will *not* ever be spent by rust-lightning, and are not at risk of your
 	/// counterparty spending them due to some kind of timeout. Thus, you need to store them
 	/// somewhere and spend them when you create on-chain transactions.
@@ -148,14 +147,20 @@ impl Writeable for Event {
 			},
 			&Event::PaymentReceived { ref payment_hash, ref payment_preimage, ref payment_secret, ref amt, ref user_payment_id } => {
 				1u8.write(writer)?;
-				payment_hash.write(writer)?;
-				payment_preimage.write(writer)?;
-				payment_secret.write(writer)?;
-				amt.write(writer)?;
-				user_payment_id.write(writer)?;
+				write_tlv_fields!(writer, {
+					(0, payment_hash),
+					(2, payment_secret),
+					(4, amt),
+					(6, user_payment_id),
+				}, {
+					(8, payment_preimage),
+				});
 			},
 			&Event::PaymentSent { ref payment_preimage } => {
 				2u8.write(writer)?;
+				write_tlv_fields!(writer, {
+					(0, payment_preimage),
+				}, {});
 				payment_preimage.write(writer)?;
 			},
 			&Event::PaymentFailed { ref payment_hash, ref rejected_by_dest,
@@ -165,24 +170,26 @@ impl Writeable for Event {
 				ref error_data,
 			} => {
 				3u8.write(writer)?;
-				payment_hash.write(writer)?;
-				rejected_by_dest.write(writer)?;
 				#[cfg(test)]
 				error_code.write(writer)?;
 				#[cfg(test)]
 				error_data.write(writer)?;
+				write_tlv_fields!(writer, {
+					(0, payment_hash),
+					(2, rejected_by_dest),
+				}, {});
 			},
 			&Event::PendingHTLCsForwardable { time_forwardable: _ } => {
 				4u8.write(writer)?;
+				write_tlv_fields!(writer, {}, {});
 				// We don't write the time_fordwardable out at all, as we presume when the user
 				// deserializes us at least that much time has elapsed.
 			},
 			&Event::SpendableOutputs { ref outputs } => {
 				5u8.write(writer)?;
-				(outputs.len() as u64).write(writer)?;
-				for output in outputs.iter() {
-					output.write(writer)?;
-				}
+				write_tlv_fields!(writer, {
+					(0, VecWriteWrapper(outputs)),
+				}, {});
 			},
 		}
 		Ok(())
@@ -192,34 +199,84 @@ impl MaybeReadable for Event {
 	fn read<R: ::std::io::Read>(reader: &mut R) -> Result<Option<Self>, msgs::DecodeError> {
 		match Readable::read(reader)? {
 			0u8 => Ok(None),
-			1u8 => Ok(Some(Event::PaymentReceived {
-					payment_hash: Readable::read(reader)?,
-					payment_preimage: Readable::read(reader)?,
-					payment_secret: Readable::read(reader)?,
-					amt: Readable::read(reader)?,
-					user_payment_id: Readable::read(reader)?,
-				})),
-			2u8 => Ok(Some(Event::PaymentSent {
-					payment_preimage: Readable::read(reader)?,
-				})),
-			3u8 => Ok(Some(Event::PaymentFailed {
-					payment_hash: Readable::read(reader)?,
-					rejected_by_dest: Readable::read(reader)?,
+			1u8 => {
+				let f = || {
+					let mut payment_hash = PaymentHash([0; 32]);
+					let mut payment_preimage = None;
+					let mut payment_secret = PaymentSecret([0; 32]);
+					let mut amt = 0;
+					let mut user_payment_id = 0;
+					read_tlv_fields!(reader, {
+						(0, payment_hash),
+						(2, payment_secret),
+						(4, amt),
+						(6, user_payment_id),
+					}, {
+						(8, payment_preimage),
+					});
+					Ok(Some(Event::PaymentReceived {
+						payment_hash,
+						payment_preimage,
+						payment_secret,
+						amt,
+						user_payment_id,
+					}))
+				};
+				f()
+			},
+			2u8 => {
+				let f = || {
+					let mut payment_preimage = PaymentPreimage([0; 32]);
+					read_tlv_fields!(reader, {
+						(0, payment_preimage),
+					}, {});
+					Ok(Some(Event::PaymentSent {
+						payment_preimage,
+					}))
+				};
+				f()
+			},
+			3u8 => {
+				let f = || {
 					#[cfg(test)]
-					error_code: Readable::read(reader)?,
+					let error_code = Readable::read(reader)?;
 					#[cfg(test)]
-					error_data: Readable::read(reader)?,
-				})),
-			4u8 => Ok(Some(Event::PendingHTLCsForwardable {
-					time_forwardable: Duration::from_secs(0)
-				})),
+					let error_data = Readable::read(reader)?;
+					let mut payment_hash = PaymentHash([0; 32]);
+					let mut rejected_by_dest = false;
+					read_tlv_fields!(reader, {
+						(0, payment_hash),
+						(2, rejected_by_dest),
+					}, {});
+					Ok(Some(Event::PaymentFailed {
+						payment_hash,
+						rejected_by_dest,
+						#[cfg(test)]
+						error_code,
+						#[cfg(test)]
+						error_data,
+					}))
+				};
+				f()
+			},
+			4u8 => {
+				let f = || {
+					read_tlv_fields!(reader, {}, {});
+					Ok(Some(Event::PendingHTLCsForwardable {
+						time_forwardable: Duration::from_secs(0)
+					}))
+				};
+				f()
+			},
 			5u8 => {
-				let outputs_len: u64 = Readable::read(reader)?;
-				let mut outputs = Vec::new();
-				for _ in 0..outputs_len {
-					outputs.push(Readable::read(reader)?);
-				}
-				Ok(Some(Event::SpendableOutputs { outputs }))
+				let f = || {
+					let mut outputs = VecReadWrapper(Vec::new());
+					read_tlv_fields!(reader, {
+						(0, outputs),
+					}, {});
+					Ok(Some(Event::SpendableOutputs { outputs: outputs.0 }))
+				};
+				f()
 			},
 			_ => Err(msgs::DecodeError::InvalidValue)
 		}
@@ -380,9 +437,49 @@ pub trait MessageSendEventsProvider {
 	fn get_and_clear_pending_msg_events(&self) -> Vec<MessageSendEvent>;
 }
 
-/// A trait indicating an object may generate events
+/// A trait indicating an object may generate events.
+///
+/// Events are processed by passing an [`EventHandler`] to [`process_pending_events`].
+///
+/// # Requirements
+///
+/// See [`process_pending_events`] for requirements around event processing.
+///
+/// When using this trait, [`process_pending_events`] will call [`handle_event`] for each pending
+/// event since the last invocation. The handler must either act upon the event immediately
+/// or preserve it for later handling.
+///
+/// Note, handlers may call back into the provider and thus deadlocking must be avoided. Be sure to
+/// consult the provider's documentation on the implication of processing events and how a handler
+/// may safely use the provider (e.g., see [`ChannelManager::process_pending_events`] and
+/// [`ChainMonitor::process_pending_events`]).
+///
+/// (C-not implementable) As there is likely no reason for a user to implement this trait on their
+/// own type(s).
+///
+/// [`process_pending_events`]: Self::process_pending_events
+/// [`handle_event`]: EventHandler::handle_event
+/// [`ChannelManager::process_pending_events`]: crate::ln::channelmanager::ChannelManager#method.process_pending_events
+/// [`ChainMonitor::process_pending_events`]: crate::chain::chainmonitor::ChainMonitor#method.process_pending_events
 pub trait EventsProvider {
-	/// Gets the list of pending events which were generated by previous actions, clearing the list
-	/// in the process.
-	fn get_and_clear_pending_events(&self) -> Vec<Event>;
+	/// Processes any events generated since the last call using the given event handler.
+	///
+	/// Subsequent calls must only process new events. However, handlers must be capable of handling
+	/// duplicate events across process restarts. This may occur if the provider was recovered from
+	/// an old state (i.e., it hadn't been successfully persisted after processing pending events).
+	fn process_pending_events<H: Deref>(&self, handler: H) where H::Target: EventHandler;
+}
+
+/// A trait implemented for objects handling events from [`EventsProvider`].
+pub trait EventHandler {
+	/// Handles the given [`Event`].
+	///
+	/// See [`EventsProvider`] for details that must be considered when implementing this method.
+	fn handle_event(&self, event: Event);
+}
+
+impl<F> EventHandler for F where F: Fn(Event) {
+	fn handle_event(&self, event: Event) {
+		self(event)
+	}
 }
